# We're trying to find a matching that maps all sets of 4 elements in
# 1..15 with the sets of subsets of 1..15 whose xor is 0, such that all
# elements are matched to elements that have no intersection with them.

import collections
 
# This class represents a directed graph using adjacency matrix representation
class BipartiteGraph:
  
    def __init__(self,left,lenright):
        self.left=left
        self.lenright=lenright
  
    def FindPath(self,unmatchedleft,rightleftmatching,leftrightmatching,parent):
        queue = collections.deque()
        visited=[False]*len(self.left)
        rightvisited=[False]*self.lenright
         
        for s in unmatchedleft:
            queue.append(s)
            visited[s] = True
         
        while queue:
            u = queue.popleft()
            for ind in self.left[u]:
              if rightvisited[ind]==False:
                rightvisited[ind]=True
                parent[ind]=u
                if rightleftmatching[ind]==None:
                  return ind
                else:
                    v=rightleftmatching[ind]
                    if visited[v] == False:
                      queue.append(v)
                      visited[v] = True
 
        return None
             
    def FordFulkerson(self):
 
        parent = [None] * self.lenright
 
        rightleftmatching=[None]*self.lenright # Initially, nothing is matched 
        leftrightmatching=[None]*len(self.left)
        unmatchedleft=set(range(len(self.left)))
 
        dest=self.FindPath(unmatchedleft, rightleftmatching, \
                           leftrightmatching, parent)

        while dest!=None:
          while parent[dest] not in unmatchedleft:
            rightleftmatching[dest]=parent[dest]
            v=leftrightmatching[parent[dest]]
            leftrightmatching[parent[dest]]=dest
            dest=v
          rightleftmatching[dest]=parent[dest]
          leftrightmatching[parent[dest]]=dest
          unmatchedleft.remove(parent[dest])
          parent = [None] * self.lenright
          dest=self.FindPath(unmatchedleft, rightleftmatching, \
                           leftrightmatching, parent)
 
        return (rightleftmatching,leftrightmatching)


def ecc(n):
  rc=0
  for i in range(15):
    rc^=(i+1)*((n>>i)&1)
  return rc

def print_binary(x):
  return ''.join([str((x>>i)&1) for i in range(14,-1,-1)])

quads=[(1<<i) + (1<<j) + (1<<k) + (1<<l) for i in range(12) \
           for j in range(i+1,13) for k in range(j+1,14) \
           for l in range(k+1,15)]

errcorr=[i for i in range(2**15) if ecc(i)==0]

quadsdict=dict([(quads[i],i) for i in range(len(quads))])

errcorrdict=dict([(errcorr[i],i) for i in range(len(errcorr))])

quadsconn=[[] for i in range(len(quads))]

errcorrconn=[[] for i in range(len(errcorr))]

for i in range(len(quads)):
  for j in range(len(errcorr)):
    if quads[i]&errcorr[j]==0:
      quadsconn[i].append(j)
      errcorrconn[j].append(i)

g=BipartiteGraph(errcorrconn,len(quads))

m=g.FordFulkerson()

for i in range(len(m[0])):
  print print_binary(quads[i])+": "+print_binary(errcorr[m[0][i]])

